// FileList.cpp

#include <afxwin.h>
#include <afxcmn.h>
#include <windowsx.h>
#include "SortDemo.rh"
#include "FileDscr.h"
#include "FileList.h"


void FileListCtrl::AddFile( const WIN32_FIND_DATA& findData )
{
    const int nPos = InsertItem(
        LVIF_TEXT | LVIF_PARAM, 0, LPSTR_TEXTCALLBACK, 0, 0, 0, 
        reinterpret_cast< LPARAM >( new FileDescr( findData ) ) );
    if ( -1 == nPos ) {
        AfxThrowMemoryException();
    }
}


void FileListCtrl::AddFiles( LPCTSTR pszDirectory )
{
    WIN32_FIND_DATA findData = { 0 };
    HANDLE h = FindFirstFile( pszDirectory, &findData );
    if ( INVALID_HANDLE_VALUE != h ) {
        AddFile( findData );
        while ( FindNextFile( h, &findData ) ) {
            AddFile( findData );
        }
        FindClose( h );
    }
    Sort();
}


void FileListCtrl::DoInitialize( void )
{
    ASSERT_VALID( this );
    
    InsertColumn( 0, _T( "File Name"  ), LVCFMT_LEFT , 110 );
    InsertColumn( 1, _T( "Short Name" ), LVCFMT_LEFT ,  90 );
    InsertColumn( 2, _T( "Size"       ), LVCFMT_RIGHT,  60 );
    InsertColumn( 3, _T( "Created"    ), LVCFMT_LEFT ,  70 );
    InsertColumn( 4, _T( "Changed"    ), LVCFMT_LEFT ,  70 );
    InsertColumn( 5, _T( "Accessed"   ), LVCFMT_LEFT ,  70 );
}


void FileListCtrl::DoDestroy() 
{
    const int nItems = GetItemCount();
    for ( int iItem = 0; iItem < nItems; ++iItem ) {
        FileDescr *pFileDescr = 
            reinterpret_cast< FileDescr * >( GetItemData( iItem ) );
        delete pFileDescr;
    }
}


static LPCTSTR GetDateFormat( const BOOL bSorting )
{
    return bSorting ? _T( "%Y%m%d%H%M%S" ) : _T( "%d-%b-%y" );
}


CString FileListCtrl::DoGetText (
    LPARAM lParam, int nColumn, BOOL bSorting ) const
{
    CString strText = _T("");
    const FileDescr *pFileDescr = 
        reinterpret_cast< const FileDescr * >( lParam );
    ASSERT( AfxIsValidAddress( pFileDescr, sizeof *pFileDescr ) );
    
    ASSERT(0 <= nColumn && nColumn < 6);
    switch ( nColumn ) {
    case 0:
        strText = pFileDescr->getFileName();
        break;
        
    case 1:
        strText = pFileDescr->getAlternateFileName();
        break;
        
    case 2:
        if ( bSorting ) {
            strText.Format( "%09lu", pFileDescr->getSize() );
        } else {
            strText.Format( "%luKB", 
                (pFileDescr->getSize() + 1024 - 1) / 1024 );
        }
        break;
        
    case 3:
        strText = pFileDescr->getCreationTime().Format( 
            ::GetDateFormat( bSorting ) );
        break;
        
    case 4:
        strText = pFileDescr->getLastAccessTime().Format( 
            ::GetDateFormat( bSorting ) );
        break;
        
    case 5:
        strText = pFileDescr->getLastWriteTime().Format( 
            ::GetDateFormat( bSorting ) );
        break;
        
    default:
        ASSERT( FALSE );
    }
    
    if ( bSorting ) {
        strText.MakeUpper();
    }
    
    return strText;
}


int FileListCtrl::DoGetSubColumn( 
    int nPrimaryColumn, int nSubColumn ) const
{
    // These arrays define the secondary sorting columns:
    // The primary column is the index. For simplicity of
    // implementation, this is also the first column here:
    static int anSortColumns[ 6 ][ 6 ] = {
        { 0, 1, 2, 3, 4, 5 }, // 0 - "File Name" 
        { 1, 0, 2, 3, 4, 5 }, // 1 - "Short Name"
        { 2, 0, 1, 3, 4, 5 }, // 2 - "Size"      
        { 3, 0, 1, 2, 4, 5 }, // 3 - "Created"   
        { 4, 0, 1, 2, 3, 5 }, // 4 - "Changed"   
        { 5, 0, 1, 2, 3, 4 }, // 5 - "Accessed"  
    };
    
#ifdef _DEBUG 
    
    // Debug code to verify that the table is correct. 
    // Make sure to execute this code whenever the table changes!
    
    #define dim(X) ( sizeof( X ) / sizeof( ( X )[ 0 ] ) )
    #define NCOLS  dim( anSortColumns )
    
    ASSERT( 0 <= nPrimaryColumn && nPrimaryColumn < NCOLS );
    ASSERT( 0 <= nSubColumn     && nSubColumn     < NCOLS );
    ASSERT( NCOLS == GetColumnCount() );
    ASSERT( NCOLS == dim( anSortColumns[ 0 ] ) );
    
    for ( int i = 0; i < NCOLS; ++i ) {
        ASSERT( i == anSortColumns[ i ][ 0 ] );
        for ( int j = 0; j < NCOLS; ++j ) {
            ASSERT( 0 <= anSortColumns[ i ][ j ] );
            ASSERT( anSortColumns[ i ][ j ] < NCOLS );
            for ( int k = j + 1; k < NCOLS; ++k ) {
                ASSERT( anSortColumns[ i ][ j ] != 
                        anSortColumns[ i ][ k ] );
            }
        }
    }
    
#endif // _DEBUG
    
    return anSortColumns[ nPrimaryColumn ][ nSubColumn ];
}

// end of file
